@@ -44,6 +44,7 @@ module Data.OrdPSQ.Internal
 
       -- * Traversals
     , map
+    , unsafeMapMonotonic
     , fold'
 
       -- * Tournament view
@@ -393,6 +394,31 @@ map f =
     goLTree (LLoser s e l k r) = LLoser s (goElem e) (goLTree l) k (goLTree r)
     goLTree (RLoser s e l k r) = RLoser s (goElem e) (goLTree l) k (goLTree r)
 
+-- | /O(n)/ Maps a function over the values and priorities of the queue.
+-- The function @f@ must be monotonic with respect to the priorities. I.e. if
+-- @x < y@, then @fst (f k x v) < fst (f k y v)@.
+-- /The precondition is not checked./ If @f@ is not monotonic, then the result
+-- will be invalid.
+{-# INLINABLE unsafeMapMonotonic #-}
+unsafeMapMonotonic
+    :: forall k p q v w.
+       (k -> p -> v -> (q, w))
+    -> OrdPSQ k p v
+    -> OrdPSQ k q w
+unsafeMapMonotonic f = goPSQ
+  where
+    goPSQ :: OrdPSQ k p v -> OrdPSQ k q w
+    goPSQ Void           = Void
+    goPSQ (Winner e l k) = Winner (goElem e) (goLTree l) k
+
+    goElem :: Elem k p v -> Elem k q w
+    goElem (E k p x) = let (p', x') = f k p x
+                       in E k p' x'
+
+    goLTree :: LTree k p v -> LTree k q w
+    goLTree Start              = Start
+    goLTree (LLoser s e l k r) = LLoser s (goElem e) (goLTree l) k (goLTree r)
+    goLTree (RLoser s e l k r) = RLoser s (goElem e) (goLTree l) k (goLTree r)
 
 -- | /O(n)/ Strict fold over every key, priority and value in the queue. The order
 -- in which the fold is performed is not specified.
